function [ R ] = compute_posterior_mppca( x, K, d, Pi, W, mu, sigmasq )
% Output: R real n x K, Pr(z_i = k)

    n = size(x, 1);
    D = size(x, 2);
    
    logR = zeros(n, K);
    for k = 1:K
        logR(:, k) = log(Pi(k)) * ones(n, 1);
        
        Wk = squeeze(W(k, :, 1:d(k))); % D x d
        if (size(Wk, 1) ~= D)
            Wk = Wk';
        end
        muk = mu(k, :);
        Sigma = sigmasq(k) * eye(D) + Wk * Wk'; % D x D
                
        Sigma = min(Sigma, 1e10 * ones(D, D));
        Sigma = max(Sigma, -1e10 * ones(D, D));      
        if (det(Sigma) == 0)
            Sigma = Sigma + 1e-6 * eye(D);
        end
        
        
        [U, S, V] = svd(Sigma);
        for i = 1:D
            logR(:, k) = logR(:, k) - (0.5 * log(S(i, i))) * ones(n, 1);
        end        
        
        A = (x - muk(ones(n, 1), :)) * Sigma^(-1); % n x D
        B = x - muk(ones(n, 1), :); % n x D
        likeli = sum(A .* B, 2); % n x 1
        logR(:, k) = logR(:, k) - 0.5 * likeli;
    end
    
    % normalize
    Z = max(logR, [], 2); % n x 1
    logR = logR - Z(:, ones(1, K)); % n x K
    eZ = sum(exp(logR), 2); % n x 1
    R = exp(logR) ./ eZ(:, ones(1, K));

end

