function [ x, z, W, mu, d ] = gen_data( n, K, D, d0, rho, sigma )
% Input:
%   n, K: int scalar, # of instances and # of components
%   D: int scalar, total dimension
%   pD: real 1 x D-1, sum to 1. probability distribution over subspace dim
%   rho, sigma: real scalar, variance within/without the subspace
%
% Output:
%   x: real n x D, the data
%   z: int n x 1, cluster assignment
%   W: real K x D x D, projection matrices. Only the leftest d_k cols
%   matter
%   mu: real K x D, offset
%   d: int K x 1, subspace dimensions

    z = zeros(n, 1);
    x = zeros(n, D);
    W = zeros(K, D, D);
    mu = zeros(K, D);  
    d = d0;
    
    % sample W and mu
    for k = 1:K
        W(k, :, 1:d(k)) = randn(D, d(k));
        mu(k, :) = randn(1, D);
    end
    
    % sample z
    rnd = rand(n, 1);
    for k = 1:K
        ind1 = ((k-1) / K * ones(n, 1) <= rnd); % n x 1
        ind2 = (rnd <= k / K * ones(n, 1)); % n x 1
        z = z + k * (ind1 .* ind2);
    end
    
    % sample x
    for k = 1:K
        dk = d(k);
        muk = mu(k, :); % 1 x D
        Wk = squeeze(W(k, :, 1:dk)); % D x dk
        if (size(Wk, 1) ~= D)
            Wk = Wk';
        end
        y = rho * randn(n, dk); % n x dk
        t = (Wk * y')' + muk(ones(n, 1), :); % n x D
        ind = (z == k * ones(n, 1)); % n x 1
        x = x + (ind(:, ones(1, D)) .* t);
    end
    
    x = x + sigma * randn(n, D);

end

