function [ z, mu ] = kmeans_my( x, K, maxT, eps )

    n = size(x, 1);
    D = size(x, 2);

    % Initialization
    mu = randn(K, D);
    
    Q = zeros(n, K);
    for k = 1:K
        muk = mu(k, :);
        Q(:, k) = sum((x - muk(ones(n, 1), :)) .^ 2, 2);
    end
    
    [min_value, z] = min(Q, [], 2);
    loss = sum(min_value);
    lastloss = 1e100;
    
    T = 0;
    while (T < maxT && abs(lastloss - loss) / lastloss > eps)
        
        T = T + 1;
        lastloss = loss;
        
        % update mu
        for k = 1:K
            ind = (z == k * ones(n, 1));
            muk = 1.0/sum(ind) * sum(ind(:, ones(1, D)) .* x);
            mu(k, :) = muk;
            Q(:, k) = sum((x - muk(ones(n, 1), :)) .^ 2, 2);
        end
        
        % update z
        [min_value, z] = min(Q, [], 2);
        loss = sum(min_value);
        
        fprintf('Iter %d: loss = %f\n', T, loss);
        
    end

end

