function [ x0, y0 ] = preprocess_hopkins( x, r, y )
% x: real d x P x F
% r: int scalar (dim. projected)
% x0: real P x r

    d = size(x, 1);
    P = size(x, 2);
    F = size(x, 3);
    
    % create the augmented matrix
    X = zeros(P, d * F);
    for i = 1:P
        for j = 1:d
            X(i, F*(j-1)+1:F*j) = squeeze(x(j, i, :));
        end
    end
    
    % Do PCA
    [coeff, score] = princomp(X);
    
    % report the top r principle directions
    x0 = zeros(P, r);
    for i = 1:P
        x0(i, :) = X(i, :) * coeff(:, 1:r);
    end
    
    % random permute the data
    perm = randperm(P);
    x1 = x0;
    y0 = zeros(P, 1);
    for i = 1:P
        x0(i, :) = x1(perm(i), :);
        y0(i) = y(perm(i));
    end

end

